-------------------------------------------------------------------------------
-- ElvUI Currency Datatext By Lockslap
-------------------------------------------------------------------------------
local E, _, V, P, G = unpack(ElvUI)
local DT = E:GetModule("DataTexts")
local L = LibStub("AceLocale-3.0"):GetLocale("ElvUI_CurrencyDatatext", false)
local EP = LibStub("LibElvUIPlugin-1.0")

local wipe = table.wipe
local sort = table.sort
local join = string.join

local displayString = ""

local function SortCurrencies()
	local temp, curHeader = {}, ""
	if GetCurrencyListSize() == 0 then return false end
	for i = 1, GetCurrencyListSize() do
		local name, isHeader, _, isUnused, isWatched, count, icon = GetCurrencyListInfo(i)
		if isHeader then
			temp[name] = {}
			curHeader = name
		else
			temp[curHeader][#temp[curHeader] + 1] = {
				["name"] = name,
				["count"] = count,
				["icon"] = icon,
				["watched"] = isWatched,
				["unused"] = isUnused,
			}
		end
	end
	return temp
end

local function OnEvent(self, event)
	local name, _, _, _, _, count, icon = GetCurrencyListInfo(E.db.currencydt.display)
	self.text:SetFormattedText(displayString, E.db.currencydt.icon == true and ("|T%s:14:14:0:0:64:64:4:60:4:60|t "):format(icon) or "", name, count)
end

local function OnEnter(self)
	local currencies = SortCurrencies()
	local currencyString = "|T%s:14:14:0:0:64:64:4:60:4:60|t |cff%s%s|r"
	local color = "ffffff"
	DT:SetupTooltip(self)
	
	if not currencies then
		DT.tooltip:AddLine(L["No Currencies Found"])
	else
		for header, items in pairs(currencies) do
			if E.db.currencydt.headers[header] then
				DT.tooltip:AddLine(header)
				for _, currency in ipairs(items) do
					if E.db.currencydt.tooltip.zeroCurrency ~= "hide" or currency.count ~= 0 then
						if currency.watched then
							color = E.db.currencydt.tooltip.colorWatched and ("%02x%02x%02x"):format(E.db.currencydt.tooltip.watchedColor.r * 255, E.db.currencydt.tooltip.watchedColor.g * 255, E.db.currencydt.tooltip.watchedColor.b * 255) or "ffffff"
						else
							if E.db.currencydt.tooltip.zeroCurrency == "gray" then
								color = currency.count == 0 and "505050" or "ffffff"
							else
								color = "ffffff"
							end
						end
						DT.tooltip:AddDoubleLine(currencyString:format(currency.icon, color, currency.name), ("|cffffff00%d|r"):format(currency.count))
					end
				end
				DT.tooltip:AddLine(" ")
			end
		end
	end

	DT.tooltip:AddDoubleLine(L["Click"], L["Toggle Currency Frame"], 1, 1, 1, 1, 1, 0)
	DT.tooltip:Show()
end

local function OnClick(self, button)
	ToggleCharacter("TokenFrame")
end

local function ValueColorUpdate(hex, r, g, b)
	displayString = join("", "%s", "|cffffffff%s:|r", " ", hex, "%d|r")
	if lastPanel ~= nil then OnEvent(lastPanel) end
end
E["valueColorUpdateFuncs"][ValueColorUpdate] = true

P["currencydt"] = {
	["display"] = 2,
	["icon"] = true,
	["tooltip"] = {
		["headers"] = true,
		["showUnused"] = true,
		["zeroCurrency"] = "gray",
		["colorWatched"] = true,
		["watchedColor"] = {r = 0, g = 1.0, b = 0},
	},
	["headers"] = {},
}

local function InjectOptions()
	if not E.Options.args.lockslap then
		E.Options.args.lockslap = {
			type = "group",
			order = -2,
			name = L["Plugins by |cff9382c9Lockslap|r"],
			args = {},
		}
	end
	
	E.Options.args.lockslap.args.currencydt = {
		type = "group",
		name = L["Currency Datatext"],
		get = function(info) return E.db.currencydt[info[#info]] end,
		set = function(info, value) E.db.currencydt[info[#info]] = value; DT:LoadDataTexts() end,
		args = {
			display = {
				type = "select",
				order = 1,
				name = L["Datatext Currency"],
				desc = L["Currency to display in the datatext."],
				values = function()
					local temp = {}
					for i = 1, GetCurrencyListSize() do
						local name, isHeader, _, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i)
						if not isHeader then
							temp[i] = name
						end
					end
					return temp
				end,
			},
			icon = {
				type = "toggle",
				order = 2,
				name = L["Show Icon"],
				desc = L["Show the display currency's icon in the datatext."],
			},
			headers = {
				type = "multiselect",
				order = 3,
				name = L["Toggle Currency Headers"],
				desc = L["Select which currency headers to show."],
				get = function(info, key) return E.db.currencydt.headers[key] end,
				set = function(info, key, value) E.db.currencydt.headers[key] = value; DT:LoadDataTexts() end,
				values = function()
					local headers = {}
					for i = 1, GetCurrencyListSize() do
						local name, isHeader = GetCurrencyListInfo(i)
						if isHeader then
							headers[name] = name
						end
					end
					return headers
				end,
			},
			tooltip = {
				type = "group",
				order = -1,
				name = L["Tooltip Options"],
				guiInline = true,
				get = function(info) return E.db.currencydt.tooltip[info[#info]] end,
				set = function(info, value) E.db.currencydt.tooltip[info[#info]] = value; DT:LoadDataTexts() end,
				args = {
					headers = {
						type = "toggle",
						order = 1,
						name = L["Show Headers"],
						desc = L["Show the currency list headers."],
					},
					zeroCurrency = {
						type = "select",
						order = 2,
						name = L["Zero Currency Handling"],
						desc = L["How to handle currencies of which you have none of, but have discovered."],
						values = {
							["none"] = L["Nothing"],
							["hide"] = L["Hide"],
							["gray"] = L["Gray Out"],
						},
					},
					colorWatched = {
						type = "toggle",
						order = 3,
						name = L["Color Watched"],
						desc = L["Color the currencies being watched.  (Shown on Backpack)"],
					},
					watchedColor = {
						type = "color",
						order = 5,
						width = "double",
						disabled = function() return not E.db.currencydt.tooltip.colorWatched end,
						name = L["Watched Color"],
						desc = L["Color to use for the watched currencies."],
						get = function()
							return E.db.currencydt.tooltip.watchedColor.r,
								E.db.currencydt.tooltip.watchedColor.g,
								E.db.currencydt.tooltip.watchedColor.b
						end,
						set = function(_, r, g, b)
							E.db.currencydt.tooltip.watchedColor.r = r
							E.db.currencydt.tooltip.watchedColor.g = g
							E.db.currencydt.tooltip.watchedColor.b = b
							DT:LoadDataTexts()
						end,
					},
				},
			},
		},
	}
end

EP:RegisterPlugin(..., InjectOptions)
DT:RegisterDatatext("Currency", {"PLAYER_ENTERING_WORLD", "CHAT_MSG_CURRENCY"}, OnEvent, nil, OnClick, OnEnter)